<?php

/**
*
* @author               Jan Kulmann <jankul@zmml.uni-bremen.de>
*
*/

// +---------------------------------------------------------------------------+
// This file is NOT part of Stud.IP
// Copyright (C) 2009 Jan Kulmann <jankul@zmml.uni-bremen.de>
// ATTENTION: Please send ALL bug reports and security leaks directly to the author.
//            Thx!
// +---------------------------------------------------------------------------+
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or any later version.
// +---------------------------------------------------------------------------+
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// +---------------------------------------------------------------------------+

require_once("lib/TwitterCacher/TwitterCacher.class.php");
require_once("classes/StudIPTwitterDB.class.php");
require_once("classes/StudIPTwitter.class.php");

class TwitterPlugin extends AbstractStudIPHomepagePlugin {

	public $factory;
	public $twitter_db;
	public $twitter;

	public function __construct(){
		parent::AbstractStudIPHomepagePlugin();
		$this->setPluginIconName('images/twitter_t.png');
		$this->factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');

		$this->twitter_db = new StudIPTwitterDB();
		$this->twitter_db->setUserId($GLOBALS['auth']->auth['uid']);

		$this->twitter = new StudIPTwitter();
		$this->twitter->setUserId($GLOBALS['auth']->auth['uid']);
		$this->twitter->setTwitterDB(&$this->twitter_db);
	}
	
	public function setPluginpath($newpath){
		parent::setPluginpath($newpath);
		$this->buildMenu();
	}
	
	public function buildMenu(){
		$tab = new PluginNavigation();
		$tab->setDisplayname(_("Twitter"));
		$tab->setActive();
		$this->setNavigation($tab); 
	}

	public function setRequestedUser($user) {
                parent::setRequestedUser($user);
                $u = $this->getRequestedUser();
                $this->setStatusShowOverviewPage(($this->twitter_db->getTwitterUsername($u->getUserid()) ? TRUE : FALSE));
		$this->twitter_db->setReqUserId($u->getUserid());
        }

	public function show() {
		$GLOBALS['perm']->check('autor');
		$this->showAdministrationPage();
	}

	public function actionShowAdministrationPage() {
		$GLOBALS['perm']->check('autor');
		$this->showAdministrationPage();
	}

	public function showAdministrationPage(){
		$GLOBALS['perm']->check('autor');
		$u = $this->getRequestedUser();
		if ($GLOBALS["auth"]->auth["uid"] !== $u->getUserid()) {
			StudIPTemplateEngine::showErrorMessage("Sie d&uuml;rfen diesen Bereich nicht betreten!");
			die;
		}
		$modus = trim($_REQUEST['twmodus']);
		if ($modus) {
			if ($modus == 'save_options') {
				$tw_username = trim($_POST['tw_username']);
				$tw_password = trim($_POST['tw_password']);
				$show_tw_friends = ($_POST['show_tw_friends'] == 'yes' ? 1 : 0);
				if ($tw_username && $tw_password) {
					$twUser = $tw_username;
					$twPass = $tw_password;

					$json = new Services_JSON();
					$tc = new TwitterCacher($twUser,$twPass);

					// I do this just to be a nice guy. So Twitter knows who we are.
					$tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

					// Read the timeline from the cache, or from Twitter.
					$response = $json->decode( $tc->verifyCredentials() );
					if (!$response) {
						StudIPTemplateEngine::showErrorMessage("Die angegebenen Accountdaten sind nicht korrekt!");
					} else {	
						$this->twitter_db->setOptions(array('tw_username'=>$tw_username,
										    'tw_password'=>$tw_password,
										    'show_tw_friends'=>$show_tw_friends));
						StudIPTemplateEngine::showSuccessMessage("Die Einstellungen wurden &uuml;bernommen.");
					}
				} else {
					$this->twitter_db->setOptions(array('tw_username'=>'',
                                                                            'tw_password'=>'',
                                                                            'show_tw_friends'=>0));
                                        StudIPTemplateEngine::showSuccessMessage("Die Einstellungen wurden &uuml;bernommen.");
				}
			}
		}
		$template = $this->factory->open('user_options');
		$template->set_attribute('options',$this->twitter_db->getOptions());
		$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
		$template->set_attribute('uri', PluginEngine::getLink($this,array(),'showAdministrationPage'));
                echo $template->render();
	}

	public function showOverview() {
		$GLOBALS['perm']->check('autor');
		$modus = trim($_REQUEST['twmodus']);
		$this->getStyle();

		if ($this->twitter_db->getOptions()) {
                        $interval = REFRESH_INTERVAL;
                } else {
                        $interval = REFRESH_INTERVAL_GUEST;
                }

		$username = isset($_GET['username']) ? $_GET['username'] : $GLOBALS["auth"]->auth["uname"];

		$new_message_tpl = "";

		if ($options = $this->twitter_db->getOptions()) {
			/*if ($modus == "create_message" && check_ticket($_POST['ticket'])) {
				$msg = trim($_POST['msg']);
				if (strlen($msg) > 0 && strlen($msg) < 141) {
					$twUser = $options[0]['tw_username'];
					$twPass = base64_decode($options[0]['tw_password']);

					$json = new Services_JSON();
					$tc = new TwitterCacher($twUser,$twPass);

					// I do this just to be a nice guy. So Twitter knows who we are.
					$tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

					// Read the timeline from the cache, or from Twitter.
					$response = $json->decode( $tc->createNewMessage($msg) );
					if ($response)
						StudIPTemplateEngine::showSuccessMessage("Die Nachricht wurde gesendet.");
					else
						StudIPTemplateEngine::showErrorMessage("Die Nachricht konnte nicht gesendet werden!");

				}
				if ($this->twitter->sendMessage($_POST['msg']))
					StudIPTemplateEngine::showSuccessMessage("Die Nachricht wurde gesendet.");
				else
					StudIPTemplateEngine::showErrorMessage("Die Nachricht konnte nicht gesendet werden!");
			}*/
			$template = $this->factory->open('create_new_message');
			$template->set_attribute('tw_username', $options[0]['tw_username']);
			//$studipticket = Seminar_Session::get_ticket();
			//$template->set_attribute('ticket', $studipticket);
			//$template->set_attribute('uri', 'about.php');
			$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
			$template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
			$new_message_tpl = $template->render();

			$template = $this->factory->open('search');
			$template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
			$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
			$search_tpl = $template->render();
		}
		
		$last_modified = $this->twitter_db->getCacheLastModified('user_timeline');
		$last_modified = ($last_modified ? $last_modified : time());
		$next_reload = $last_modified + $interval;
		$template = $this->factory->open('timeline_container');
                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
		$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
		$template->set_attribute('new_message_tpl', $new_message_tpl);
		$template->set_attribute('search_tpl', $search_tpl);
		$template->set_attribute('req_username', $username);
		$template->set_attribute('refresh', $interval * 1000);
		$template->set_attribute('interval', $interval);
		$template->set_attribute('next_reload', date('H:i',$next_reload));
		$template->set_attribute('style', $this->getStyle(TRUE));
                echo $template->render();

	}

	public function actionAjaxDispatch(){
                ob_end_clean();
                if (!array_key_exists('ajax_cmd', $_REQUEST)) die;
                switch($_REQUEST['ajax_cmd']) {
                        case 'get_tweets':
				$req_username = trim($_REQUEST['req_username']);
				$req_userid = get_userid($req_username);

				$twPass = "";
				$twUser = "";

				$options = $this->twitter_db->getOptions();
				if ($options) {
					$twUser = $options[0]['tw_username'];
					$twPass = base64_decode($options[0]['tw_password']);
				}
				if ($req_userid !== $GLOBALS['auth']->auth['uid']) {
					$twUser = $this->twitter_db->getTwitterUsername($req_userid);
				}
				$this->twitter_db->setReqUserId($req_userid);

				$json = new Services_JSON();
				$tc = new TwitterCacher($twUser,$twPass);
				$tc->setTimelineUser(($req_userid == $GLOBALS['auth']->auth['uid'] ? FALSE : $twUser));
				$tc->setTwitterDB(&$this->twitter_db);

				// I do this just to be a nice guy. So Twitter knows who we are.
				$tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

				// Read the timeline from the cache, or from Twitter.
				$timeline = $json->decode( $tc->getUserTimeline() );
				
				if ($timeline) {
					$template = $this->factory->open('user_timeline');
			                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
					$template->set_attribute('timeline', $timeline);
		        	        echo $template->render();
				} else {
					echo "Sorry, no tweets found....";
				}
				break;
			case 'do_search':
				$q= trim($_POST['search_str']);
				if (strlen($q) > 0 && strlen($q) < 141) {
					$twPass = "";
	                                $twUser = "";

        	                        $options = $this->twitter_db->getOptions();
                	                if ($options) {
                        	                $twUser = $options[0]['tw_username'];
                                	        $twPass = base64_decode($options[0]['tw_password']);
                               		}

					$json = new Services_JSON();
					$tc = new TwitterCacher($twUser,$twPass);

					// I do this just to be a nice guy. So Twitter knows who we are.
					$tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

					// Read the timeline from the cache, or from Twitter.
					$response = $json->decode( $tc->doSearch($q) );
					if ($response) {
						$template = $this->factory->open('search_result');
				                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
						$template->set_attribute('results', $response->results);
				                echo $template->render();
					} else {
						echo "Sorry, no tweets found....";
					}
				}
				break;
			case 'send_message':
                                $msg = trim(escapeshellcmd(basename(utf8_decode($_REQUEST['msg_value']))));
                                if ($this->twitter->sendMessage($msg))
                                        StudIPTemplateEngine::showSuccessMessage("Die Nachricht wurde gesendet.");
                                else
                                        StudIPTemplateEngine::showErrorMessage("Die Nachricht konnte nicht gesendet werden!");
                                break;
			default: ;
                }
                die();
        }

	private function getStyle($ret=FALSE) {
		if (!$ret)
			echo "<link href=\"".$this->getPluginURL()."/css/twitter_style.css\" rel=\"stylesheet\" type=\"text/css\" media=\"all\">\n";
		else
			return "<link href=\"".$this->getPluginURL()."/css/twitter_style.css\" rel=\"stylesheet\" type=\"text/css\" media=\"all\">\n";
	}

}

?>
