<?php
/**
* @author               Jan Kulmann <jankul@zmml.uni-bremen.de>
*
*/

// +---------------------------------------------------------------------------+
// This file is NOT part of Stud.IP
// Copyright (C) 2009 Jan Kulmann <jankul@zmml.uni-bremen.de>
// ATTENTION: Please send ALL bug reports and security leaks directly to the author.
//            Thx!
// +---------------------------------------------------------------------------+
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or any later version.
// +---------------------------------------------------------------------------+
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// +---------------------------------------------------------------------------+

require_once("lib/TwitterCacher/TwitterCacher.class.php");
require_once("classes/StudIPTwitterDB.class.php");
require_once("classes/StudIPTwitter.class.php");

class TwitterPortalPlugin extends AbstractStudIPPortalPlugin {

	private $factory;
        private $twitter_db;
        private $twitter;

	public function __construct(){
		parent::AbstractStudIPPortalPlugin();
		$this->setPluginIconName('images/twitter_t.png');
		$this->factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');

                $this->twitter_db = new StudIPTwitterDB();
                $this->twitter_db->setUserId($GLOBALS['auth']->auth['uid']);
		$this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);

		$this->twitter = new StudIPTwitter();
                $this->twitter->setUserId($GLOBALS['auth']->auth['uid']);
                $this->twitter->setTwitterDB(&$this->twitter_db);
	}
	
	public function setPluginpath($newpath){
		parent::setPluginpath($newpath);
		$this->buildMenu();
	}

	public function buildMenu(){
		$tab = new PluginNavigation();
		$tab->setDisplayname(_("Twitter"));
		$tab->setActive();
	}

	public function getNavigation() {
		return $this->navigation;
	}

	public function getPluginName() {
		return "Twitter Friends & Walls";
	}
	
	public function hasAdministration() {
		return FALSE;
	}

	public function showOverview($unauthorizedview = TRUE) {
		$GLOBALS['perm']->check('autor');

		if ($options = $this->twitter_db->getOptions()) {
                        $interval = REFRESH_INTERVAL;
                } else {
                        $interval = REFRESH_INTERVAL_GUEST;
                }

		$last_modified = $this->twitter_db->getCacheLastModified('friends_timeline');
		$last_modified = ($last_modified ? $last_modified : time());
                $next_reload = $last_modified + $interval;

		$new_message_tpl = "";
		if ($options) {
			$template = $this->factory->open('create_new_message');
        	        $template->set_attribute('tw_username', $options[0]['tw_username']);
                	$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
                	$template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
	                $new_message_tpl = $template->render();

			$template = $this->factory->open('search');
                        $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
                        $template->set_attribute('img_uri', $this->getPluginURL().'/images/');
                        $search_tpl = $template->render();
		}

		$template = $this->factory->open('timeline_container');
                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
                $template->set_attribute('img_uri', $this->getPluginURL().'/images/');
                $template->set_attribute('req_username', $GLOBALS["auth"]->auth["uname"]);
		$template->set_attribute('refresh', $interval * 1000);
		$template->set_attribute('interval', $interval);
		$template->set_attribute('search_tpl', $search_tpl);
		$template->set_attribute('new_message_tpl', $new_message_tpl);
		$template->set_attribute('next_reload', date('H:i',$next_reload));
                $template->set_attribute('style', $this->getStyle(TRUE));
                echo $template->render();
	}

	public function actionAjaxDispatch(){
                ob_end_clean();
                if (!array_key_exists('ajax_cmd', $_REQUEST)) die;
                switch($_REQUEST['ajax_cmd']) {
                        case 'get_tweets':
                                $options = $this->twitter_db->getOptions();
                                $twUser = $options[0]['tw_username'];
                                $twPass = base64_decode($options[0]['tw_password']);
                                $this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);

                                $json = new Services_JSON();
                                $tc = new TwitterCacher($twUser,$twPass);
                                $tc->setTimelineMode('friends_timeline');
                                $tc->setTwitterDB(&$this->twitter_db);

                                // I do this just to be a nice guy. So Twitter knows who we are.
                                $tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

                                // Read the timeline from the cache, or from Twitter.
                                $timeline = $json->decode( $tc->getUserTimeline() );

                                $template = $this->factory->open('user_timeline');
                                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
                                $template->set_attribute('timeline', $timeline);
                                $body = $template->render();
				$template = $this->factory->open('index_overview');
                                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
				$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
				$template->set_attribute('body', $body);
				echo $template->render();
                                break;
			case 'get_wall':
				$wall_id = trim(escapeshellcmd(basename($_REQUEST['wall_value'])));
				if ($wall_id) {
					$this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);
					$wall = $this->twitter_db->getWall($wall_id);
					if (!$wall) die();
					else $q = $wall['wall_keyword'];
					$twPass = "";
                                        $twUser = "";

                                        $options = $this->twitter_db->getOptions();
                                        if ($options) {
                                                $twUser = $options[0]['tw_username'];
                                                $twPass = base64_decode($options[0]['tw_password']);
                                        }

                                        $json = new Services_JSON();
                                        $tc = new TwitterCacher($twUser,$twPass);
					$tc->setTwitterDB(&$this->twitter_db);
					$tc->setTimelineMode('wall_timeline');
					$tc->setWallKeyword($q);

                                        // I do this just to be a nice guy. So Twitter knows who we are.
                                        $tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

                                        // Read the timeline from the cache, or from Twitter.
                                        $response = $json->decode( $tc->getUserTimeline() );
                                        if ($response) {
                                                $template = $this->factory->open('search_result_wall');
                                                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
                                                $template->set_attribute('wall_keyword', $q);
                                                $template->set_attribute('results', $response->results);
                                                echo $template->render();
                                        } else {
                                                echo "Sorry, no tweets found....";
                                        }
       				}
                                break;
			case 'get_walls':
				$this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);
				$walls = $this->twitter_db->getWalls();
				if ($walls) {
					$out = "";
					foreach ($walls as $w) {
						$template = $this->factory->open('wall_head');
						$template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
						$template->set_attribute('img_uri', $this->getPluginURL().'/images/');
						$template->set_attribute('w', $w);
						$template->set_attribute('rechte', ($w['user_id'] == $GLOBALS['auth']->auth['uid'] ? TRUE : FALSE));
						$out .= $template->render();
					}
					echo $out;
				} else echo "";
				break;
			case 'create_wall':
				$wall_keyword = trim(escapeshellcmd(basename(utf8_decode($_REQUEST['wall_value']))));
				if ($wall_keyword) {
					$this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);
					$this->twitter_db->saveWall($wall_keyword);
				}
				echo "New Wall";
				break;
			case 'remove_wall':
				$wall_id = trim(escapeshellcmd(basename($_REQUEST['wall_value'])));
				if ($wall_id) {
					$this->twitter_db->setReqUserId($GLOBALS['auth']->auth['uid']);
					$this->twitter_db->removeWall($wall_id);
				}
			case 'send_message':
				$msg = trim(escapeshellcmd(basename(utf8_decode($_REQUEST['msg_value']))));
				if ($this->twitter->sendMessage($msg))
                                        StudIPTemplateEngine::showSuccessMessage("Die Nachricht wurde gesendet.");
                                else
                                        StudIPTemplateEngine::showErrorMessage("Die Nachricht konnte nicht gesendet werden!");
				break;
			case 'do_search':
                                $q= trim($_POST['search_str']);
                                if (strlen($q) > 0 && strlen($q) < 141) {
                                        $twPass = "";
                                        $twUser = "";

                                        $options = $this->twitter_db->getOptions();
                                        if ($options) {
                                                $twUser = $options[0]['tw_username'];
                                                $twPass = base64_decode($options[0]['tw_password']);
                                        }

                                        $json = new Services_JSON();
                                        $tc = new TwitterCacher($twUser,$twPass);

                                        // I do this just to be a nice guy. So Twitter knows who we are.
                                        $tc->setUserAgent("Mozilla/5.0 (compatible; TwitterCacher/1.0;)");

                                        // Read the timeline from the cache, or from Twitter.
                                        $response = $json->decode( $tc->doSearch($q) );
                                        if ($response) {
                                                $template = $this->factory->open('search_result');
                                                $template->set_attribute('ajax_uri', PluginEngine::getLink($this,array(),'ajaxDispatch'));
                                                $template->set_attribute('results', $response->results);
                                                echo $template->render();
                                        } else {
                                                echo "Sorry, no tweets found....";
                                        }
                                }
                                break;
			default: ;
                }
                die();
        }


        public function hasUnauthorizedView() { return FALSE; }

        public function hasAuthorizedView() { 
		$options = $this->twitter_db->getOptions();
		return ($options && $options[0]['show_tw_friends'] == 1 ? TRUE : FALSE); 
	}

	private function getStyle($ret=FALSE) {
                if (!$ret)
                        echo "<link href=\"".$this->getPluginURL()."/css/twitter_style.css\" rel=\"stylesheet\" type=\"text/css\" media=\"all\">\n";
                else
                        return "<link href=\"".$this->getPluginURL()."/css/twitter_style.css\" rel=\"stylesheet\" type=\"text/css\" media=\"all\">\n";
        }

}

?>
